"""
Tests for carta_envoy.launchers
"""
from pathlib import Path
from unittest.mock import MagicMock, patch

# pylint: disable=E0401, R0201

from carta_envoy.utilities import (
    CARTA_URL_REPLACE_TEXT,
    CARTA_HTML_FILENAME,
    CARTA_TEMPLATE,
)
from carta_envoy.launchers import CartaLauncher


UI_URL = "http://localhost:4444/workspaces"
test_settings = {
    "timeout": 1,
    "carta_path": "/fake/path/to/nowhere",
    "redis_host": "noplace.nrao.edu",
    "redis_port": "6397",
    "redis_password": "password",
    "reverse_proxy": "black-abyss.nrao.edu",
    "data_location": ".",
    "useCarta": False,
    "notification_url": "http://notification:3458",
    "workflow_url": "http://workflow:3456",
    "wf_request_id": -1,
}

launcher = CartaLauncher(settings=test_settings)

SUBPROCESS_COMMAND_PATCH = "carta_envoy.launchers.subprocess.Popen"

CARTA_HTML_TEST_PATH = "/path/to/nowhere"
WRAPPER_URL = "http://localhost:4444/workspaces/carta/requests/-1/html"

BACK_END_PORT = 7777
FRONT_END_PORT = 6464


class TestCartaLauncher:
    """
    Exercise various CartaLauncher abilities.

    """

    def test_prepare_carta(self):
        """
        Make sure Redis is prepared for CARTA.

        :return:
        """
        with patch("carta_envoy.connect.RedisConnect.prepare_redis") as redis_prep:
            launcher.prepare_for_carta()
            assert redis_prep.call_count == 1

    def test_launch(self):
        """
        Confirm that CartaLauncher behaves as expected.

        :return:
        """
        with patch("carta_envoy.launchers.CartaLauncher.prepare_for_carta") as prep:
            with patch("carta_envoy.launchers.CartaLauncher.run_carta") as carta:
                launcher.launch()
                assert prep.call_count == 1
                assert carta.call_count == 1

    def test_dont_run_carta(self):
        """
        Test that CARTA doesn't run if it isn't set to
        """
        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            launcher.run_carta(
                path_to_carta="fake_carta_path",
                timeout_minutes=1,
                file_browser_path=Path(),
                front_end_port=FRONT_END_PORT,
                back_end_port=BACK_END_PORT,
                carta_url="carta_url",
            )
            assert mock_subprocess.call_count == 0

    @patch("carta_envoy.launchers.CartaLauncher.create_frame_html")
    def test_run_carta_with_aat(self, mock_frame):
        """
        Test that CARTA runs successfully with setup and teardown
        """
        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "false"
        mock_archive_connect = MagicMock()
        launcher.archive_connect = mock_archive_connect

        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            launcher.run_carta(
                path_to_carta="fake_carta_path",
                timeout_minutes=1,
                file_browser_path=Path(),
                front_end_port=FRONT_END_PORT,
                back_end_port=BACK_END_PORT,
                carta_url="carta_url",
            )
            assert mock_subprocess.call_count == 1
            mock_subprocess.assert_called_with(
                [
                    "fake_carta_path",
                    f"--port={BACK_END_PORT}",
                    f"--fport={FRONT_END_PORT}",
                    "--folder=.",
                    "--root=.",
                ],
                preexec_fn=None,
                stdin=-1,
                stdout=-1,
                stderr=-1,
            )
            assert mock_frame.call_count == 1
            mock_archive_connect.send_carta_url_to_rh.assert_called_with("carta_url")

    @patch("carta_envoy.launchers.CartaLauncher.create_frame_html")
    def test_run_carta_with_ws(self, mock_frame):
        """
        Test that CARTA runs successfully with setup and teardown
        """
        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "true"
        mock_notification_connect = MagicMock()
        launcher.notification = mock_notification_connect

        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
            launcher.run_carta(
                path_to_carta="fake_carta_path",
                timeout_minutes=1,
                file_browser_path=Path(),
                front_end_port=FRONT_END_PORT,
                back_end_port=BACK_END_PORT,
                carta_url="carta_url",
            )
            assert mock_subprocess.call_count == 1
            mock_subprocess.assert_called_with(
                [
                    "fake_carta_path",
                    f"--port={BACK_END_PORT}",
                    f"--fport={FRONT_END_PORT}",
                    "--folder=.",
                    "--root=.",
                ],
                preexec_fn=None,
                stdin=-1,
                stdout=-1,
                stderr=-1,
            )
            assert mock_frame.call_count == 1
            mock_notification_connect.send_session_ready.assert_called_with("carta_url")

    @patch("pathlib.Path.exists")
    def test_generates_carta_html(self, mock_path):
        """
        Test that we can make a nice HTML page containing the CARTA URL in a frame
        :return:
        """

        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "true"
        mock_notification_connect = MagicMock()
        launcher.notification = mock_notification_connect

        with patch("carta_envoy.launchers.CartaLauncher.create_frame_html") as mock_carta_html:
            with patch(SUBPROCESS_COMMAND_PATCH):
                launcher.run_carta(
                    path_to_carta=test_settings["carta_path"],
                    timeout_minutes=1,
                    file_browser_path=Path(test_settings["data_location"]),
                    front_end_port=FRONT_END_PORT,
                    back_end_port=BACK_END_PORT,
                    carta_url="carta url",
                )

        assert mock_carta_html.call_count == 1

    def test_serves_carta_wrapper(self):
        """
        Can we create HTML containing the CARTA URL in a frame and serve it up?

        :return:
        """
        launcher.settings["useCarta"] = True
        launcher.settings["send_ready"] = "true"
        mock_notification_connect = MagicMock()
        launcher.notification = mock_notification_connect

        fake_carta_path = "fake_carta_path"

        # 3. pretend to launch CARTA
        with patch("pathlib.Path.write_text"):
            with patch("pathlib.Path.read_text", return_value=CARTA_TEMPLATE):
                with patch("pathlib.Path.exists", return_value=True):
                    with patch("pathlib.Path.is_dir", return_value=True):
                        with patch(SUBPROCESS_COMMAND_PATCH) as mock_subprocess:
                            launcher.run_carta(
                                path_to_carta=fake_carta_path,
                                timeout_minutes=1,
                                file_browser_path=Path(CARTA_HTML_TEST_PATH),
                                front_end_port=FRONT_END_PORT,
                                back_end_port=BACK_END_PORT,
                                carta_url="carta url",
                            )
                            assert mock_subprocess.call_count == 1
                            mock_subprocess.assert_called_with(
                                [
                                    fake_carta_path,
                                    f"--port={BACK_END_PORT}",
                                    f"--fport={FRONT_END_PORT}",
                                    "--folder=" + CARTA_HTML_TEST_PATH,
                                    "--root=" + CARTA_HTML_TEST_PATH,
                                ],
                                preexec_fn=None,
                                stdin=-1,
                                stdout=-1,
                                stderr=-1,
                            )
                            mock_notification_connect.send_session_ready.assert_called_with("carta url")

        carta_html = Path(CARTA_HTML_TEST_PATH) / CARTA_HTML_FILENAME
        expected_html = CARTA_TEMPLATE.replace(CARTA_URL_REPLACE_TEXT, WRAPPER_URL)
        with patch("pathlib.Path.read_text", return_value=expected_html):
            assert WRAPPER_URL in carta_html.read_text()
