""" Miscellaneous manifest-building tests """

import logging
import sys
from pathlib import Path

# pylint: disable=E0401, E0402, R1721, W0611, W0621
import shutil

from ingest_envoy.ingestion_manifest import (
    IngestionManifest,
    IngestionManifestBuilder,
    find_manifest,
)
from ingest_envoy.manifest_components import (
    INGESTION_ARTIFACTS_NAME,
    TARFILE_EXT,
)
from ingest_envoy.utilities import ScienceProductType, Telescope
from .conftest import (
    ingest_path,
    populate_fake_evla_cal_ingest_path,
    populate_fake_tmpx_ratuqh_ingest_path,
    ANCILLARY_PRODUCTS,
    OUTPUT_GROUP_SCIENCE_PRODUCT_ANCILLARIES,
)

logger = logging.getLogger(IngestionManifest.__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def test_entry_point_for_evla_cal(ingest_path: Path):
    """
    Confirm that the ingestion launcher entrypoint kicks off production of ingestion manifest

    :param ingest_path: fake tmp ingestion path
    :return:
    """
    locator = "uid://evla/calibration/but-we-had-this-for-lunch-yesterday"
    populate_fake_evla_cal_ingest_path(ingest_path)
    sp_tar = [file for file in ingest_path.iterdir() if file.name.endswith(".tar")][0]
    assert sp_tar

    builder = IngestionManifestBuilder(
        telescope=Telescope.EVLA.value,
        locator=locator,
        sp_type=ScienceProductType.EVLA_CAL.value,
        staging_source_dir=ingest_path,
    )
    builder.build()
    ingestion_files = [file for file in ingest_path.iterdir()]

    # there should be one ingestion manifest....
    manifest_file = find_manifest(ingest_path)
    assert manifest_file

    # ...and an artifacts tar, and the science products tar we started with
    assert sp_tar in ingestion_files
    artifact_tars = [
        file
        for file in ingestion_files
        if file.name.startswith(INGESTION_ARTIFACTS_NAME) and file.name.endswith(TARFILE_EXT)
    ]
    assert len(artifact_tars) == 1


def test_entry_point_for_image(ingest_path: Path):
    """
    Confirm that the ingestion launcher entrypoint kicks off production
    of standard imaging manifest

    :param ingest_path: staging directory
    :return:
    """

    locator = "uid://evla/calibration/ea93dae5-3495-47fa-887d-4be2852f5f14"
    populate_fake_tmpx_ratuqh_ingest_path(ingest_path)

    # we should be starting out with various image manifest input files
    # and CASA byproducts, a random file, and -not- the image ingestion
    # manifest yet to be created
    expected_file_count_before = (
        len(ANCILLARY_PRODUCTS) + len(OUTPUT_GROUP_SCIENCE_PRODUCT_ANCILLARIES) + 1
    )
    ingestion_files_before = [file for file in ingest_path.iterdir()]
    assert len(ingestion_files_before) == expected_file_count_before

    IngestionManifestBuilder(
        telescope=Telescope.EVLA.value,
        locator=locator,
        sp_type=ScienceProductType.IMAGE.value,
        staging_source_dir=ingest_path,
    ).build()

    # there should be one ingestion manifest....
    manifest_file = find_manifest(ingest_path)
    assert manifest_file

    # ...and everything we started with, plus a new ingestion artifacts tar
    ingestion_files_after = [file for file in ingest_path.iterdir()]
    if len(ingestion_files_after) != expected_file_count_before + 2:
        for file in ingestion_files_before:
            if file not in ingestion_files_after:
                print(f">>> {file.name} missing after manifest build")
        for file in ingestion_files_after:
            if file not in ingestion_files_before:
                print(f">>> {file.name} present after manifest build")

    assert len(ingestion_files_after) == expected_file_count_before + 2
    shutil.rmtree(ingest_path)
